<?php

namespace Drupal\block_ajax;

use Drupal\block\Entity\Block;
use Drupal\Core\Block\BlockPluginInterface;
use Drupal\Core\Block\BlockManager;
use Drupal\Core\Cache\CacheTagsInvalidator;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Security\TrustedCallbackInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Ajax Blocks Service.
 */
class AjaxBlocks implements TrustedCallbackInterface {

  use StringTranslationTrait;

  /**
   * The config factory object.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;

  /**
   * The current user service.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The cache tags invalidator.
   *
   * @var \Drupal\Core\Cache\CacheTagsInvalidator
   */
  protected $cacheTagsInvalidator;

  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * The block manager service.
   *
   * @var \Drupal\Core\Block\BlockManager
   */
  protected $blockManager;

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * Ajax Blocks Constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactory $config_factory
   *   The config factory object.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   The current user object.
   * @param \Drupal\Core\Cache\CacheTagsInvalidator $cache_tags_invalidator
   *   The cache tags invalidator service.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date formatter service.
   * @param \Drupal\Core\Block\BlockManager $block_manager
   *   The block manager service.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   */
  public function __construct(
    ConfigFactory          $config_factory,
    AccountProxyInterface  $current_user,
    CacheTagsInvalidator   $cache_tags_invalidator,
    DateFormatterInterface $date_formatter,
    BlockManager           $block_manager,
    RendererInterface      $renderer
  ) {
    $this->configFactory = $config_factory;
    $this->currentUser = $current_user;
    $this->cacheTagsInvalidator = $cache_tags_invalidator;
    $this->dateFormatter = $date_formatter;
    $this->blockManager = $block_manager;
    $this->renderer = $renderer;
  }

  /**
   * Check to see if user has proper permissions/access rights.
   *
   * @return bool
   *   Returns access right.
   */
  public function hasAccess(): bool {
    return $this->currentUser->hasPermission('administer ajax blocks');
  }

  /**
   * Check if given block is configured as an Ajax block.
   *
   * @param \Drupal\Core\Block\BlockPluginInterface $block
   *   Block plugin.
   *
   * @return bool
   *   Returns TRUE if current block is AJAX block.
   */
  public function isAjaxBlock(BlockPluginInterface $block): bool {
    $conf = $block->getConfiguration();
    return !empty($conf['block_ajax']['is_ajax']);
  }

  /**
   * Check if given block is configured to load via Lazy builder.
   *
   * @param \Drupal\Core\Block\BlockPluginInterface $block
   *   Block plugin.
   *
   * @return bool
   *   Returns TRUE if current block is loaded via Lazy builder.
   */
  public function isLazyBlock(BlockPluginInterface $block): bool {
    $conf = $block->getConfiguration();
    return !empty($conf['block_ajax']['is_lazy']);
  }

  /**
   * Render Ajax block using lazy builder.
   *
   * @param string $block_id
   *
   * @return array
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function renderLazyAjaxBlock(string $block_id): array {
    $build = [];

    // Load block.
    $block = Block::load($block_id);

    // Construct and render the block.
    if ($block && $plugin = $block->getPlugin()) {
      $build = $this->blockManager->createInstance($plugin->getPluginId(), $plugin->getConfiguration())
        ->build();
    }

    // Return render array for block.
    return $build;
  }

  /**
   * Invalidate Ajax block cache tags.
   *
   * @return void
   */
  public function invalidateAjaxBlocks() {
    $this->cacheTagsInvalidator->invalidateTags(['block_ajax']);
  }

  /**
   * Get max age options.
   *
   * @return array
   */
  public function getMaxAgeOptions(): array {
    $periods = [
      0,
      60,
      180,
      300,
      600,
      900,
      1800,
      2700,
      3600,
      10800,
      21600,
      32400,
      43200,
      86400,
    ];
    $periods = array_map([
      $this->dateFormatter,
      'formatInterval',
    ], array_combine($periods, $periods));
    $periods[0] = $this->t('No caching');

    return $periods;
  }

  /**
   * Get Ajax defaults for $.ajax().
   *
   * @return array
   * @todo Make admin configurable.
   */
  public function getAjaxDefaults(): array {
    return [
      'type' => 'POST',
      'timeout' => 10000,
      'async' => TRUE,
      'cache' => FALSE,
      'dataType' => 'json',
    ];
  }

  /**
   * @inheritDoc
   */
  public static function trustedCallbacks(): array {
    return [
      'renderLazyAjaxBlock',
    ];
  }

}
